local M = {}

local droneVolume = 0
local dronePitch = 0
local speed = 0
local oldspeed = 0
local counter = 0
local accel = 0
local droneEmitter = 0
local run = 0
local driver = false

local function getNode() -- check if node 'dr1' exists, checks only once when the car spawns.
  if run == 1 then 
    return true -- returns true or false immediately to not constantly search nodes
  elseif run == 2 then
    return false
  end
  for i, node in ipairs(v.data.nodes) do
    if node.name == "dr1" then 
      droneEmitter = i 
      run = 1
      return true
    end
  end
  run = 2
  return false
end

function setCameraStatus(status) 
  driver = status == "driver" and true or false
end

local function updateGFX(dt)

  if getNode() then 
    counter = counter + 1 
    accel = counter == 4 and math.abs((speed - oldspeed) / (dt * 100)) or accel -- using velocity over time to get acceleration
    speed = electrics.values.airflowspeed * 3.6 
    oldspeed = counter == 4 and speed or oldspeed
    counter = counter >= 4 and 0 or counter
  
    dronePitch = math.max(((speed / 80) + 0.5), 1)
    dronePitch = accel + dronePitch -- both linear speed and acceleration factored into dronePitch
    dronePitch = math.min(dronePitch, 3.3)

    droneVolume = math.max(((speed / 500) + 0.05), 0.65)
    droneVolume = math.min(droneVolume, 0.3)

    obj:queueGameEngineLua("be:getObjectByID("..tostring(objectId).."):queueLuaCommand('setCameraStatus(\"'..tostring(core_camera.getActiveCamName())..'\")')") -- dont know why this works but i am not going to change it

    droneVolume = driver == true and droneVolume * 0.2 or droneVolume -- quieter if in cockpit view

    obj:setVolumePitch(drone1, droneVolume, dronePitch)
    obj:setVolumePitch(drone2, droneVolume, dronePitch)
    obj:setVolumePitch(drone3, droneVolume, dronePitch)
    obj:setVolumePitch(drone4, droneVolume, dronePitch)
    obj:setVolumePitch(drone_wind, 0.3, 10)
  end
end

local function onInit() 
  if getNode() then
    drone1 = obj:createSFXSource("/vehicles/common/techeyy_drone/sounds/drone1.mp3", "AudioDefaultLoop3D", "drone1", droneEmitter)
    drone2 = obj:createSFXSource("/vehicles/common/techeyy_drone/sounds/drone2.mp3", "AudioDefaultLoop3D", "drone2", droneEmitter)
    drone3 = obj:createSFXSource("/vehicles/common/techeyy_drone/sounds/drone3.mp3", "AudioDefaultLoop3D", "drone3", droneEmitter)
    drone4 = obj:createSFXSource("/vehicles/common/techeyy_drone/sounds/drone4.mp3", "AudioDefaultLoop3D", "drone4", droneEmitter)
    drone_wind = obj:createSFXSource("event:>Vehicle>Aero>windbuffet", "AudioDefaultLoop3D", "drone_wind", droneEmitter)
  end
end

-- public interface
M.onInit = onInit
M.updateGFX = updateGFX
M.onReset = onReset
return M